#####################################################################
#####    Tree-Structured Clustering in Fixed Effects Models     #####
#####              Electronic Supplement                        #####
#####################################################################
#####     Author:  Moritz Berger                                #####
#####################################################################
#####     Content: R-Code of the simulation                     #####
#####              in Section 6.4 Extreme Settings              ##### 
#####################################################################


### Function one_sim() ###

## Input 
# n_ni:   number of observation units / number of measurements 
# k:      true size of clusters (m_0)
# rho:    correlation between intercepts and the first covariate 
# random: distribution of the unit-specific intercepts 
# seed:   integer passed to set.seed() 

## Description  
# The function 
# - generates one data set composed of the response y 
#   and the matrix of covariates DM_kov
# - estimates the six models: GFM, TSC, GMM, FINA, FINB, PENL
# - calculates MSEs, FPR/FNR, number of Clusters 
#
# The function requires the self-implemented R add-on package "structree", 
# as well as the R add-on packages "lme4", "flexmix" and "gvcm.cat". 
# In addition the R scripts "FP_FN.R" and "flexmixFix.R" are used.  

## Output  
# Matrix with 6 rows and 5 columns (one row per method). 
# One row contains MSE of the intercepts, MSE of the linear term, 
# FPR, FNR and the number of clusters. 

############################


one_sim <- function(n_ni,
                    k,
                    rho,
                    random,
                    seed){
  
  # variance components 
  sigma_bi <-1
  sigma_e  <-3
  
  # initialization
  if(n_ni==2004){
    n  <- 200      # number of observation units
    ni <- 4        # number of reapeted measurements
    min_b <- n/20  # minimal number of observation units in the 
                   # outer nodes of the tree 
                   # (i.e. at the edge of the value range of the intercepts)
  }
  if(n_ni==1008){
    n  <- 100
    ni <- 8
    min_b <- n/50 
  }
  if(n_ni==4020){
    n  <- 40
    ni <- 20 
    min_b <- NULL
  }
  if(n_ni==2040){
    n  <- 20
    ni <- 40
    min_b <- NULL 
  }
  
  if(k==0){
    k <- n 
  }

  ### data ### 
  
  set.seed(seed)
  xbinom <- rbinom(n*ni,1,0.5)  # binomial for all observations
  xnorm  <- rnorm(n*ni,0,1)     # standard-normal for all observations
  x0     <- rep(1:n,each=ni)    # second level: n 
  
  # random effect for second level
  if(random=="normal"){
    bi <- rnorm(n,0,sigma_bi)            
  } 
  if(random=="chi2"){
    bi <- rchisq(n,sigma_bi^2/2)-(sigma_bi^2/2) 
  }
  
  # correlation structure
  Sigma      <- matrix(c(sigma_bi^2,rho*sigma_bi*1,rho*sigma_bi*1,1),nrow=2,ncol=2)
  Sigma_sqrt <- t(chol(Sigma))
  y1     <- (bi[x0]-Sigma_sqrt[1,2]*xnorm)/Sigma_sqrt[1,1]
  xnorm  <- Sigma_sqrt[2,1]*y1+Sigma_sqrt[2,2]*xnorm
  
  # clustering 
  if(k<n){
    help  <- rep(1:k,each=(n/k))
    means <- sapply(split(sort(bi),help),mean)
    means <- rep(means,each=(n/k))
    bi[order(bi)] <- means
  } 
  
  # response and design
  betabinom <- 2
  betanorm  <- 2 
  
  eta <- bi[x0]+xbinom*betabinom+xnorm*betanorm
  
  DM_kov <- data.frame(x0,xbinom,xnorm)
  DM_kov$x0 <- as.factor(DM_kov$x0)
  secondlevel <- "x0"
  
  y <- eta+rnorm(n*ni,0,sigma_e)
  data <- cbind(y=y,DM_kov)
  
  ###
  
  # initialization
  results <- matrix(NA,nrow=6,ncol=5)
  rownames(results) <- c("Fixed","TSC","Mixed","FinitA","FinitB","gvcm.cat")
  colnames(results) <- c("MSE_bi","MSE_linear","FPR","FNR","ncluster")
  
  ### GFM ###
  
  design_fm <- sapply(1:n, function(j) ifelse(DM_kov$x0 ==j,1,0))
  colnames(design_fm) <- paste0("x0_",1:n)
  data_fm <- data.frame(y,design_fm,xbinom,xnorm)
  
  # model fit 
  mod_full <- glm(y~.-1,data=data_fm)
  
  # estimates
  bi_hat_fixed <- coef(mod_full)[1:n]
  x_hat_fixed  <- tail(coef(mod_full),2)
  
  results[1,1]    <- sum((bi-bi_hat_fixed)^2)/n # MSE - intercepts
  results[1,2]    <- sum((c(betabinom,betanorm)-x_hat_fixed)^2)/2 # MSE - linear term
  results[1,3]    <- 1
  results[1,4]    <- 0
  results[1,5]    <- n
  
  ### TSC ###
  
  require("structree")
  source("./functions/FP_FN.R")
  
  # maximal number of splits
  if(n==200){
    st <- 50 
  } else{
    st <- 20
  }
  
  # model fit 
  mod_tree <- structree(y~tr(1|x0)+xbinom+xnorm,data=data,family=gaussian, 
                        stop_criterion="pvalue",alpha=0.05,splits_max=st,
                        min_border=min_b,trace=FALSE)
  
  # estimates
  opt_tree    <- mod_tree$opts+1
  bi_hat_tree <- mod_tree$beta_hat[["int|x0"]][,opt_tree]
  x_hat_tree  <- tail(mod_tree$coefs_end,2)
  
  results[2,1]    <- sum((bi-bi_hat_tree)^2)/n # MSE - intercepts
  results[2,2]    <- sum((c(betabinom,betanorm)-x_hat_tree)^2)/2 # MSE - linear term
  results[2,3]    <- FP_FN(bi,bi_hat_tree)[1] # FPR
  results[2,4]    <- FP_FN(bi,bi_hat_tree)[2] # FNR
  results[2,5]    <- opt_tree
  
  ### GMM ###
  
  require("lme4")
  
  # model fit 
  mod_mixed <- lmer(y~xbinom+xnorm+(1|x0),data=data)
  
  # estimates
  bi_hat_mixed <- unlist(ranef(mod_mixed)$x0+fixef(mod_mixed)[1])
  x_hat_mixed   <- fixef(mod_mixed)[2:3]
  
  results[3,1]    <- sum((bi-bi_hat_mixed)^2)/n # MSE - intercepts
  results[3,2]    <- sum((c(betabinom,betanorm)-x_hat_mixed)^2)/2 # MSE - linear term
  results[3,3]    <- 1
  results[3,4]    <- 0
  results[3,5]    <- n 
  
  ### FINA/FINB ###
  
  require("flexmix")
  source("./functions/flexmixFix.R")
  
  # model fit 
  mod_finit <- initFlexmix(y~1|x0,model=FLXMRglmfix(fixed=~xbinom+xnorm,varFix=TRUE),
                           control=list(minprior=0),k=1:20,data=data,nrep=3,
                           verbose=FALSE)
  
  # selection 
  mod_finitAIC <- getModel(mod_finit,which="AIC")
  mod_finitBIC <- getModel(mod_finit,which="BIC")
  
  # estimates
  help <- seq(1,n*ni,by=ni)
  partition_hat_finitA <- mod_finitAIC@cluster[help]
  partition_hat_finitB <- mod_finitBIC@cluster[help]
  bi_hat_finitA        <- parameters(mod_finitAIC)[3,][partition_hat_finitA]
  bi_hat_finitB        <- parameters(mod_finitBIC)[3,][partition_hat_finitB]
  x_hat_finitA         <- parameters(mod_finitAIC)[1:2,1]
  x_hat_finitB         <- parameters(mod_finitBIC)[1:2,1]
  
  results[4,1]    <- sum((bi-bi_hat_finitA)^2)/n # MSE - intercepts
  results[4,2]    <- sum((c(betabinom,betanorm)-x_hat_finitA)^2)/2 # MSE - linear term
  results[4,3]    <- FP_FN(bi,bi_hat_finitA)[1] # FPR
  results[4,4]    <- FP_FN(bi,bi_hat_finitA)[2] # FNR
  results[4,5]    <- length(unique(mod_finitAIC@cluster))
  
  results[5,1]    <- sum((bi-bi_hat_finitB)^2)/n # MSE - intercepts
  results[5,2]    <- sum((c(betabinom,betanorm)-x_hat_finitB)^2)/2 # MSE - linear term
  results[5,3]    <- FP_FN(bi,bi_hat_finitB)[1] # FPR
  results[5,4]    <- FP_FN(bi,bi_hat_finitB)[2] # FNR
  results[5,5]    <- length(unique(mod_finitBIC@cluster))
  
  ### PENL ###
  
  if(n<100){
    
    require("gvcm.cat")
    
    repeat{
      # model fit 
      mod_gvcm  <- try(gvcm.cat(y~v(1,x0)+xnorm+xbinom,data=data,
                          control=cat_control(tuning.criterion="deviance",adapted.weights=TRUE,K=5,
                                                c=10^(-9)),silent=TRUE))
      if(class(mod_gvcm)[1]!="try-error"){
        break
      }
    }
      
    # estimates
    bi_hat_gvcm <- mod_gvcm$coefficients[1:n]
    x_hat_gvcm  <- tail(mod_gvcm$coefficients,2)
    
    results[6,1]    <- sum((bi-bi_hat_gvcm)^2)/n # MSE - intercepts
    results[6,2]    <- sum((c(betabinom,betanorm)-x_hat_gvcm)^2)/2 # MSE - linear term
    results[6,3]    <- FP_FN(bi,bi_hat_gvcm)[1] # FPR
    results[6,4]    <- FP_FN(bi,bi_hat_gvcm)[2] # FNR
    results[6,5]    <- length(unique(bi_hat_gvcm))
  }
  
  return(results)
}

############################

### Execution ### 

# All the results of one scenario (seed=1:100) are stored in seperate .rda-files in the folder
# "./simulation6.4/raw_results6.4". k=0 corresponds to m_0=n and k=1 corresponds to m_0=1. 
# For example, "no_0_0_2004.rda" contains the results of the setting 
# with normally distributed intercepts, rho=0, m_0=n, n=200 and n_i=4. 


## Example 1: n=200, n_i=4, m_0=n, normal intercepts, rho=0
one_sim(n_ni=2004, k=0, rho=0, random="normal", seed=42)

# result is stored in 
load("./simulation6.4/raw_results6.4/no_0_0_2004.rda")
no_0_0_2004[[42]]


## Example 2: n=20, n_i=40, m_0=1, normal intercepts, rho=0.8
one_sim(n_ni=2040, k=1, rho=0.8, random="normal", seed=23)

# result is stored in 
load("./simulation6.4/raw_results6.4/no_0.8_1_2040.rda")
no_0.8_1_2040[[23]]


## Whole run of one simulation setting (Note: should take approx. 2h)
res <- list()
for(i in 1:100){
  res[[i]] <- one_sim(n_ni=2004, k=0, rho=0, random="normal", seed=i)
}
# save res as .rda-file 
# sc <- paste(c("no","0","0","2004"),collapse="_")
# assign(sc,res)
# save(list=sc, file = paste0("./simulation6.4/raw_results6.4/",sc,".rda"))


# The whole simulation was executed on a Linux Cluster 
# by use of the R add-on package "BatchJobs". 
# The original call was the following:

# library("BatchJobs")
# r <- makeRegistry(id="simulation6.4",file.dir="./Simulation/simulation6.4")
# r <- loadRegistry(file.dir="./Simulation/simulation6.4")
# 
# batchExpandGrid(r,one_sim,
#                 seed=1:100,
#                 k=c(0,1),
#                 n_ni=c(2004,1008,4020,2040),
#                 rho=c(0,0.8), 
#                 more.args=list(random="normal"))
# 
# submitJobs(r,1:1600)













