#####################################################################
#####    Tree-Structured Clustering in Fixed Effects Models     #####
#####              Electronic Supplement                        #####
#####################################################################
#####     Author:  Moritz Berger                                #####
#####################################################################
#####     Content: R-Code of the simulated bootstrap            #####
#####              confidence intervals in Section 4            ##### 
#####################################################################


### Function one_sim() ###

## Input 
# seed: integer passed to set.seed() 
# B:    number of bootstrap samples    

## Description  
# The function 
# - generates one data set composed of the response y 
#   and the matrix of covariates DM_kov
# - estimates the four models: GFM, TSC, GMM, FINB
# - calculates the 95% CI based on the B bootstrap samples 
#
# The function requires the self-implemented R add-on package "structree", 
# as well as the R add-on packages "lme4" and "flexmix". 
# In addition the R script "flexmixFix.R" is used.  

## Output  
# Matrix with 2 rows and 4 columns (one column per method).
# One column contains the estimated .025 and .975 quantile for beta.   

############################


one_sim <- function(seed,B){

  set.seed(seed)
    
  ### data ### 
  
  # initialization 
  n       <- 30
  ni      <- 50
  sigma_e <- 4

  x  <- rbinom(n*ni,1,0.5)  # binomial covariate
  x0 <- rep(1:n,each=ni)
  bi <- c(rep(32.384,2),rep(33.434,7),rep(33.904,6),rep(34.517,9),rep(34.999,7),rep(36.264,4))
  bx <- -2
  
  # response and design 
  eta <- bi[x0]+x*bx
  
  DM_kov <- data.frame(x0,x)
  DM_kov$x0 <- as.factor(DM_kov$x0)
  secondlevel <- "x0"
  
  y <- eta+rnorm(n*ni,0,sigma_e)
  
  # initialization 
  results <- matrix(NA,ncol=4,nrow=B)
  seeds <- sample(200000:500000,B,replace=FALSE)
  
  for(b in 1:B){
    
    # draw boostrap sample that contains each observation unit 
    ok <- FALSE
    while(!ok){
      set.seed(seeds[b])
      obs    <- sample(1:nrow(DM_kov),nrow(DM_kov),replace=TRUE)
      y_boot      <- y[obs]
      DM_kov_boot <- DM_kov[obs,]
      if(!(any(table(DM_kov_boot$x0)==0))){
        ok <- TRUE
      } else{
        seed <- seed+1
      }
    }
    data <- cbind(y=y_boot,DM_kov_boot)

    ###
    
    ### GFM ###
    
    design_fm <- sapply(1:n, function(j) ifelse(DM_kov_boot$x0 ==j,1,0))
    colnames(design_fm) <- paste0("x0_",1:n)
    data_fm <- data.frame(y=y_boot,design_fm,x=DM_kov_boot$x)
    
    # model fit 
    mod_full <- glm(y~.-1,data=data_fm)
    
    results[b,1] <- tail(coef(mod_full),1) 
    
    ### Tree ###
    
    require("structree")
    
    # model fit 
    mod_tree <- structree(y~tr(1|x0)+x, data=data, family=gaussian, 
                          stop_criterion="pvalue", alpha=0.05, trace=FALSE)
    
    results[b,2] <- tail(mod_tree$coefs_end,1)
  
    ### GMM ###
    
    require("lme4")
    
    # model fit 
    mod_mixed <- lmer(y~x+(1|x0),data=data)
    
    results[b,3] <- fixef(mod_mixed)[2]
    
    ### FINB ###
    
    require("flexmix")
    source("./functions/flexmixFix.R")
    
    # model fit 
    mod_finit <- initFlexmix(y~1|x0,model=FLXMRglmfix(fixed=~x,varFix=TRUE),
                             control=list(minprior=0),k=1:20,data=data,nrep=3,
                             verbose=FALSE)
    mod_finitBIC <- getModel(mod_finit,which="BIC")
    results[b,4] <- parameters(mod_finitBIC)[1,1]
  }
  
  # compute quantiles 
  KI_boot <- apply(results,2, function(x) quantile(x,probs=c(0.025,0.975)))
  
  return(KI_boot)
}

############################

### Execution ### 

# The results of (seed=1:300, B=1000) are stored in simulation_boot.rda-files in the folder
# "./simulationBoot/raw_resultsBoot". (Note: takes many days without use of parallel computing!)

## One examplary run with 5 bootstrap samples 
one_sim(seed=10,B=1)
 

# The whole simulation was executed on a Linux Cluster 
# by use of the R add-on package "BatchJobs". 
# The original call was the following: 

# library("BatchJobs")
# r <- makeRegistry(id="simulationBoot",file.dir="./Simulation/simulationBoot")
# r <- loadRegistry(file.dir="./Simulation/simulationBoot")
#
# batchExpandGrid(r,one_sim,seed=1:300,more.args=list(B=1000))
# submitJobs(r,1:300,max.retries=20)







