#####################################################################
#####    Tree-Structured Clustering in Fixed Effects Models     #####
#####              Electronic Supplement                        #####
#####################################################################
#####     Author:  Moritz Berger                                #####
#####################################################################
#####     Content: R-Code of the simulation                     #####
#####              in Appendix B Extension                      ##### 
#####################################################################


### Function one_sim() ###

## Input 
# seed: integer passed to set.seed() 

## Description  
# The function 
# - generates one data set composed of the response y 
#   and the matrix of covariates DM_kov
# - estimates the five models: TSC, GMM, FINA, FINB, PENL
# - calculates MSEs, FPR/FNR, number of Clusters 
#
# The function requires the self-implemented R add-on package "structree", 
# as well as the R add-on packages "lme4", "flexmix" and "gvcm.cat". 
# In addition the R scripts "FP_FN.R" and "flexmixFix.R" are used.  
# In order to use the modified version of the algorithm that allows for 
# constant covariates, the argument 'constant_covs' has to be set to TRUE, 
# when calling structree, see line 107. 

## Output  
# Matrix with 5 rows and 5 columns (one row per method). 
# One row contains MSE of the intercepts, MSE of the linear term, 
# FPR, FNR and the number of clusters. 

############################

one_sim <- function(seed){
  
  # initialization
  sigma_bi <- 1
  sigma_e  <- 3
  n   <- 20
  ni  <- 40
  k   <- 5
  rho <- 0 
  random <- "normal"
  
  ### data ### 
  
  set.seed(seed)
  xbinom  <- rbinom(n*ni,1,0.5)        # binomial for all observations
  xnorm   <- rnorm(n*ni,0,1)           # standard-normal for all observations
  xnorm_s <- rep(rnorm(n,0,1),each=ni) # standard-normal constant over second level 
  x0      <- rep(1:n,each=ni)          # second level: n 
  
  # random effect for second level
  if(random=="normal"){
    bi <- rnorm(n,0,sigma_bi)            
  } 
  if(random=="chi2"){
    bi <- rchisq(n,sigma_bi^2/2)-(sigma_bi^2/2) 
  }
  
  # correlation structure 
  Sigma      <- matrix(c(sigma_bi^2,rho*sigma_bi*1,rho*sigma_bi*1,1),nrow=2,ncol=2)
  Sigma_sqrt <- t(chol(Sigma))
  y1     <- (bi[x0]-Sigma_sqrt[1,2]*xnorm)/Sigma_sqrt[1,1]
  xnorm  <- Sigma_sqrt[2,1]*y1+Sigma_sqrt[2,2]*xnorm
  
  # clustering
  if(k<n){
    help  <- rep(1:k,each=(n/k))
    means <- sapply(split(sort(bi),help),mean)
    means <- rep(means,each=(n/k))
    bi[order(bi)] <- means
  }
  
  # response and design 
  betabinom <- 1
  betanorm  <- 1 
  betanorm_s<- 1 
  
  eta <- bi[x0]+xbinom*betabinom+xnorm*betanorm+xnorm_s*betanorm_s
  
  DM_kov <- data.frame(x0,xbinom,xnorm,xnorm_s)
  DM_kov$x0 <- as.factor(DM_kov$x0)
  secondlevel <- "x0"
  
  y <- eta+rnorm(n*ni,0,sigma_e)
  data <- cbind(y=y,DM_kov)
  
  ###
  
  results <- matrix(NA,nrow=5,ncol=5)
  rownames(results) <- c("TSC","Mixed","FinitA","FinitB","gvcm.cat")
  colnames(results) <- c("MSE_bi","MSE_linear","FPR","FNR","ncluster")
  
  ### TSC ###
  
  require("structree")
  source("./functions/FP_FN.R")
  
  # model fit 
  mod_tree <- structree(y~tr(1|x0)+xbinom+xnorm+xnorm_s,data=data,family=gaussian, 
                        stop_criterion="pvalue",alpha=0.05,splits_max=20,
                        constant_covs=TRUE,trace=FALSE) # constant_covs has to be set to TRUE
  
  # estimates
  opt_tree    <- mod_tree$opts+1
  bi_hat_tree <- mod_tree$beta_hat[["int|x0"]][,opt_tree]
  x_hat_tree  <- tail(mod_tree$coefs_end,3)
  
  results[1,1]    <- sum((bi-bi_hat_tree)^2)/n # MSE - intercepts
  results[1,2]    <- sum((c(betabinom,betanorm,betanorm_s)-x_hat_tree)^2)/2 # MSE - linear term
  results[1,3]    <- FP_FN(bi,bi_hat_tree)[1] # FPR
  results[1,4]    <- FP_FN(bi,bi_hat_tree)[2] # FNR
  results[1,5]    <- opt_tree
  
  ### GMM ###
  
  require("lme4")
  
  # model fit
  mod_mixed <- lmer(y~xbinom+xnorm+xnorm_s+(1|x0),data=data)
  
  # estimates
  bi_hat_mixed <- unlist(ranef(mod_mixed)$x0+fixef(mod_mixed)[1])
  x_hat_mixed   <- fixef(mod_mixed)[2:4]
  
  results[2,1]    <- sum((bi-bi_hat_mixed)^2)/n # MSE - intercepts
  results[2,2]    <- sum((c(betabinom,betanorm,betanorm_s)-x_hat_mixed)^2)/2 # MSE - linear term
  results[2,3]    <- 1
  results[2,4]    <- 0
  results[2,5]    <- n 
  
  ### FINA/FINB ###
  
  require("flexmix")
  source("./functions/flexmixFix.R")
  
  # model fit
  mod_finit <- initFlexmix(y~1|x0,model=FLXMRglmfix(fixed=~xbinom+xnorm+xnorm_s,varFix=TRUE),
                           control=list(minprior=0),k=1:20,data=data,nrep=3,
                           verbose=FALSE)
  
  # selection 
  mod_finitAIC <- getModel(mod_finit,which="AIC")
  mod_finitBIC <- getModel(mod_finit,which="BIC")
  
  # estimates
  help <- seq(1,n*ni,by=ni)
  partition_hat_finitA <- mod_finitAIC@cluster[help]
  partition_hat_finitB <- mod_finitBIC@cluster[help]
  bi_hat_finitA        <- parameters(mod_finitAIC)[4,][partition_hat_finitA]
  bi_hat_finitB        <- parameters(mod_finitBIC)[4,][partition_hat_finitB]
  x_hat_finitA         <- parameters(mod_finitAIC)[1:3,1]
  x_hat_finitB         <- parameters(mod_finitBIC)[1:3,1]
  
  results[3,1]    <- sum((bi-bi_hat_finitA)^2)/n # MSE - intercepts
  results[3,2]    <- sum((c(betabinom,betanorm,betanorm_s)-x_hat_finitA)^2)/2 # MSE - linear term
  results[3,3]    <- FP_FN(bi,bi_hat_finitA)[1] # FPR
  results[3,4]    <- FP_FN(bi,bi_hat_finitA)[2] # FNR
  results[3,5]    <- length(unique(mod_finitAIC@cluster))
  
  results[4,1]    <- sum((bi-bi_hat_finitB)^2)/n # MSE - intercepts
  results[4,2]    <- sum((c(betabinom,betanorm,betanorm_s)-x_hat_finitB)^2)/2 # MSE - linear term
  results[4,3]    <- FP_FN(bi,bi_hat_finitB)[1] # FPR
  results[4,4]    <- FP_FN(bi,bi_hat_finitB)[2] # FNR
  results[4,5]    <- length(unique(mod_finitBIC@cluster))
  
  ### PENL ###
  
  if(n<100){
    
    require("gvcm.cat")
    
    # model fit
    mod_gvcm  <- try(gvcm.cat(y~v(1,x0)+xnorm+xbinom+xnorm_s,data=data,
                              control=cat_control(tuning.criterion="deviance",adapted.weights=TRUE,K=5,
                                                  c=10^(-9),lambda.upper=3,lambda.lower=0)),silent=TRUE)
    
    if(class(mod_gvcm)[1]!="try-error"){
    
      # estimates
      bi_hat_gvcm <- mod_gvcm$coefficients[1:n]
      x_hat_gvcm  <- tail(mod_gvcm$coefficients,3)
    
      results[5,1]    <- sum((bi-bi_hat_gvcm)^2)/n # MSE - intercepts
      results[5,2]    <- sum((c(betabinom,betanorm,betanorm_s)-x_hat_gvcm)^2)/2 # MSE - linear term
      results[5,3]    <- FP_FN(bi,bi_hat_gvcm)[1] # FPR
      results[5,4]    <- FP_FN(bi,bi_hat_gvcm)[2] # FNR
      results[5,5]    <- length(unique(bi_hat_gvcm))
    
    }
  }
  
  return(results)
}

############################

### Execution ### 

# All the results (seed=1:200) are stored in simulation_Extension.rda in the folder
# "./simulationExtension/raw_resultsExtension". 
# The simulation was run 200 times, because PENL in many cases does not work. 

## One exemplary run 
one_sim(seed=13)

# result is stored in 
load("./simulationExtension/raw_resultsExtension/simulation_Extension.rda")
res[[13]]


# The whole simulation was executed on a Linux Cluster 
# by use of the R add-on package "BatchJobs". 
# The original call was the following: 

# library("BatchJobs")
# r <- makeRegistry(id="simulationExtension",file.dir="./Simulation/simulationExtension")
# r <- loadRegistry(file.dir="./Simulation/simulationExtension")
# 
# batchExpandGrid(r,one_sim,
#                 seed=1:200)
# 
# submitJobs(r,1:200)











